<?php

use PrestaShop\Module\ConotoxiaPay\Repository\PaymentRepository;
use PrestaShop\Module\ConotoxiaPay\Repository\RefundRepository;

class ConotoxiaPayNotificationModuleFrontController extends ModuleFrontController
{
    /**
     * @var CKPL\Pay\Pay
     */
    protected $sdk;

    public function initContent(): void
    {
        parent::initContent();
        try {
            $module = new ConotoxiaPay();
            $this->setTemplate($module->buildTemplatePath('notification'));
            $this->sdk = $module->initializeConotoxiaPay();
            if ($_SERVER['REQUEST_METHOD'] === 'POST') {
                $input = file_get_contents('php://input');
                $notification = $this->sdk->notification()->getNotification($input);
                if ($notification->isPaymentNotification()) {
                    $this->processIncomingPayment($notification);
                    header('HTTP/1.1 200 OK');
                    echo 'OK';
                } elseif ($notification->isRefundNotification()) {
                    $this->processIncomingRefund($notification);
                    header('HTTP/1.1 200 OK');
                    echo 'OK';
                } else {
                    header('HTTP/1.1 400 Bad Request', true, 400);
                    echo 'Unsupported notification';
                }
            } else {
                header('HTTP/1.1 405 Method Not Allowed', true, 405);
                echo 'Method Not Allowed';
            }
        } catch (Exception $e) {
            header('HTTP/1.1 400 Bad Request', true, 400);
            echo $e->getMessage();
        }
        ob_flush();
    }

    /**
     * @param CKPL\Pay\Notification\Payment\PaymentNotificationInterface $notification
     *
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    private function processIncomingPayment($notification): void
    {
        if ($notification->isCompleted()) {
            $order = new Order($notification->getExternalPaymentId());
            if (OrderState::invoiceAvailable(2)) {
                $order->setInvoice(true);
            }
            $order->setCurrentState(2);
        } elseif ($notification->isRejected() || $notification->isCancelled()) {
            $order = new Order($notification->getExternalPaymentId());
            $order->setCurrentState(6);
        }
        PaymentRepository::updatePaymentStatus($notification->getPaymentId(), $notification->getStatus());
    }

    /**
     * @param CKPL\Pay\Notification\Refund\RefundNotificationInterface $notification
     * @throws PrestaShopDatabaseException
     */
    private function processIncomingRefund($notification): void
    {
        RefundRepository::updateRefundStatus($notification->getRefundId(), $notification->getStatus());
    }
}