<?php

use PrestaShop\Module\ConotoxiaPay\Repository\PaymentRepository;

class ConotoxiaPayPaymentModuleFrontController extends ModuleFrontController
{
    public $ssl = true;

    /**
     * @var ConotoxiaPay
     */
    private $conpay;

    /**
     * @see FrontController::initContent()
     */
    public function initContent()
    {
        parent::initContent();
        $this->conpay = new ConotoxiaPay();

        $this->context->smarty->assign($this->getTemplateVariables());
        $this->setTemplate($this->conpay->buildTemplatePath('payment_execution'));

        if ($this->module->isConfigurationCompleted() && Tools::getValue('payByConPay')) {
            $this->pay($this->module->initializeConotoxiaPay());
        }
    }

    public function postProcess()
    {
        $cart = $this->context->cart;
        if ($cart->id_customer == 0 ||
            $cart->id_address_delivery == 0 ||
            $cart->id_address_invoice == 0 ||
            !$this->module->active) {
            Tools::redirect('index.php?controller=order&step=1');
        }
    }

    /**
     * @return array
     */
    private function getTemplateVariables()
    {
        $presentedCart = $this->cart_presenter->present($this->context->cart);
        return array(
            'subtotalShipping' => $presentedCart['subtotals']['shipping'],
            'subtotalProducts' => $presentedCart['subtotals']['products'],
            'total' => $presentedCart['totals']['total'],
            'products_count' => $this->context->cart->nbProducts(),
            'smallLogo' => $this->conpay->getLogo()
        );
    }

    /**
     * @param CKPL\Pay\Pay $sdk
     *
     * @throws \CKPL\Pay\Exception\ClientException
     * @throws \CKPL\Pay\Exception\Exception
     */
    private function pay($sdk)
    {
        $this->conpay->validateOrder(
            $this->context->cart->id, 12,
            $this->context->cart->getOrderTotal(true, Cart::BOTH),
            $this->conpay->displayName,
            null,
            array(),
            (int)$this->context->cart->id_currency,
            false,
            $this->context->cart->secure_key,
            Context::getContext()->shop->id
                ? new Shop((int)Context::getContext()->shop->id) : null
        );

        $order = new Order($this->conpay->currentOrder);

        try {
            $paymentBuilder = $sdk->payments()->createPaymentBuilder();
            $paymentAmount = $this->buildPaymentAmount($paymentBuilder);

            $createdPayment = $sdk->payments()->makePayment(
                $paymentBuilder
                    ->setExternalPaymentId($order->id)
                    ->setAmount($paymentAmount)
                    ->setDescription($order->id)
                    ->setIntegrationPlatform($this->getShopVersions())
                    ->getPayment()
            );
        } catch (\CKPL\Pay\Exception\ClientException | \CKPL\Pay\Exception\Exception $e) {
            PrestaShopLogger::addLog('ConotoxiaPay - payment creation problem: ' . $e->getMessage(), 3, $e->getCode(), 'Order', $order->id, true);
            throw $e;
        }

        PaymentRepository::createPayment($order->id, $createdPayment->getPaymentId());
        $order->addOrderPayment($paymentAmount->getValue(), $this->l('Conotoxia Pay'), $createdPayment->getPaymentId());

        Tools::redirect($createdPayment->getApproveUrl());
    }

    /**
     * @param CKPL\Pay\Definition\Payment\Builder\PaymentBuilderInterface $paymentBuilder
     *
     * @return CKPL\Pay\Definition\Amount\AmountInterface
     * @throws \CKPL\Pay\Exception\Definition\AmountException
     */
    private function buildPaymentAmount($paymentBuilder)
    {
        $totalAmount = $this->context->cart->getOrderTotal(true, Cart::BOTH);
        $currency = new Currency((int)$this->context->cart->id_currency);

        return $paymentBuilder->createAmountBuilder()
            ->setValue(number_format($totalAmount, $currency->precision, '.', ''))
            ->setCurrency($currency->iso_code)
            ->getAmount();
    }

    /**
     * @return string
     */
    private function getShopVersions()
    {
        return 'PRESTA=' . _PS_VERSION_ . ';PLUGIN=' . $this->conpay->version;
    }
}
