<?php

declare(strict_types=1);

namespace CKPL\Pay\Notification\Refund;

/**
 * Class RefundNotification.
 *
 * @package CKPL\Pay\Notification\Refund
 */
class RefundNotification implements RefundNotificationInterface
{
    /**
     * @var string
     */
    protected $refundId;

    /**
     * @var string|null
     */
    protected $externalRefundId;

    /**
     * @var string
     */
    protected $code;

    /**
     * @type string
     */
    const REFUND_STATUS_NEW = 'NEW';

    /**
     * @type string
     */
    const REFUND_STATUS_COMPLETED = 'COMPLETED';

    /**
     * @type string
     */
    const REFUND_STATUS_PROCESSING = 'PROCESSING';

    /**
     * @type string
     */
    const REFUND_STATUS_PENDING = 'PENDING';

    /**
     * Status constructor.
     *
     * @param string $refundId
     * @param string $code
     * @param string $externalRefundId
     */
    public function __construct(string $refundId, string $code, string $externalRefundId = null)
    {
        $this->refundId = $refundId;
        $this->code = $code;
        $this->externalRefundId = $externalRefundId;
    }

    /**
     * @return string
     */
    public function getRefundId(): string
    {
        return $this->refundId;
    }

    /**
     * @return string|null
     */
    public function getExternalRefundId(): ?string
    {
        return $this->externalRefundId;
    }

    /**
     * @return string
     */
    public function getStatus(): string {
        return $this->code;
    }

    /**
     * @return bool
     */
    public function isNew(): bool
    {
        return $this->code === static::REFUND_STATUS_NEW;
    }

    /**
     * @return bool
     */
    public function isCompleted(): bool
    {
        return $this->code === static::REFUND_STATUS_COMPLETED;
    }

    /**
     * @return bool
     */
    public function isProcessing(): bool
    {
        return $this->code === static::REFUND_STATUS_PROCESSING;
    }

    /**
     * @return bool
     */
    public function isPending(): bool
    {
        return $this->code === static::REFUND_STATUS_PENDING;
    }

    /**
     * @return bool
     */
    public function isPaymentNotification(): bool
    {
        return false;
    }

    /**
     * @return bool
     */
    public function isRefundNotification(): bool
    {
        return true;
    }
}
